# -*- coding: utf-8 -*-
"""core.connection.py

Handling remote database connection

Attributes:

  - RESULT_FETCH(tuple): Indicates ``fetchall`` or ``fetchone``
                         for DBAPI2 libraries cursor.

Example usage:

    >>>
    >>>


"""

__author__ = "Papavassileiou Vassilis"

__version__ = "0.1"

__revision__ = "0.1.0"

__date__ = "26-08-2014"

__copyright__ = "Copyright (c) 2014-2015 Geotag Aeroview Ltd"

__license__ = "GNU General Public License (GPL), Version 3"


from pgsuite.rpc import pg_query_builder

__all__ = ['pg_query', ]

RESULT_FETCH = (
    (True, 'fetchall'),
    (False, 'fetchone'),
)


def pg_query(session, api_type, api_code, params=None, data_format=dict,
             fetch_all=True):
    """Execute a Postgresql stored procedure or view and retrieve data in
    specified format.

        Args:
          - session (instance)       : A Postgresql connection instance.
          - api_type (str)           : The pg_nspname ('view', 'function').
          - api_name (str)           : The name of the API (i.e public.view).
          - params (list - optional) : All necessary parameters if needed.
          - format (str)             : Declares the return format type.
          - fetch_all (boolean)      : Indicates cursor fetchall or fetchone`

        Returns:
            - List objects with Postgresql API call or function data in
              requested format.
            - Available formats: (dict, models.Model)

        Raises:
            - PgAPINotFoundError: if api_name doen't exists on Database.
            - InvalidPgOperationError: if api_type not in ['view', 'function'].


    """
    with session.cursor() as cursor:
        cursor.execute(pg_query_builder(api_type, api_code, params))
        return [data_format(**dict(
            zip([col.name for col in cursor.description], row)
        ))
            for row in getattr(cursor, dict(RESULT_FETCH)[fetch_all])()]


if __name__ == "__main__":
    import doctest
    doctest.testmod()