# -*- coding: utf-8 -*-
"""core.rpc module

Core utilities for executing Remote Procedures on Postgresql Server.

Attributes:

    PG_TYPES (tuple) : A tuple with Postgresql avaiolable RPC types
"""

__author__ = "Papavassileiou Vassilis"

__version__ = "0.1"

__revision__ = "0.1.0"

__date__ = "19-08-2014"

__copyright__ = "Copyright (c) 2014-2015 Papavasileiou Vassilis."

__license__ = "GNU General Public License (GPL), Version 3"


from errors import (
    InvalidParamsFormatException, RPCTypeError
)


PG_TYPES = ('function', 'view', 'query')


def _pg_params_formatter(params):
    """Formats parameters for Postgresql function execution.

        ``Example usage``:

            >>> params = ['pav', 27, [1, 2, 3]]
            >>> print _pg_params_formatter(params)
            'pav', 27, '{1, 2, 3}'

        Args:
          - params (list) : List of function parameters.

        Returns:
          - Formatted String representation of the function parameters.

        Raises:

          - InvalidParamsFormatError: If params is not hashable.

    """

    if not params:
        return None

    if not isinstance(params, list):
        raise InvalidParamsFormatException('Params must be type <list>.')

    formatted_params = str(list(params))[1:-1]
    return formatted_params.replace("[", "'{").replace("]", "}'")


def pg_query_builder(pg_type, pg_code, params=None):
    """Formats as string a RPC for a Postgresql function of view.

    ``Example usage``:

        >>> params = ['vpapavasil@cmail.com', 'pav', 27]
        >>> print pg_query_builder('function', 'public.test_fn', params=None)
        SELECT * FROM public.test_fn()

        >>> print pg_query_builder('view', 'public.test_view', params=None)
        SELECT * FROM public.test_view

        Args:
          - pg_type (str) : The type of RPC (view, or function)
          - pg_code (str) : The full name or the RPC (i.e "public.my_view")
                            or query code
          - params (str)  : The parameters of the RPC ("function" type)

        Returns:
          - A string representation of the RPC in SQL syntax.

        Raises:
            - RPCTypeError: If pg_type not in PG_TYPES.
    """

    if pg_type not in PG_TYPES:
        raise RPCTypeError(PG_TYPES)

    if pg_type == 'query':
        return pg_code

    formatted_rpc = """SELECT * FROM {}""".format(pg_code)

    if pg_type is 'function':
        formatted_rpc += "({})".format(_pg_params_formatter(params) or '')

    return formatted_rpc


if __name__ == '__main__':
    import doctest
    doctest.testmod()
